package ru.ulstu.extractor.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import ru.ulstu.extractor.model.Branch;
import ru.ulstu.extractor.model.mvc.FilterForm;
import ru.ulstu.extractor.model.mvc.RepoForm;
import ru.ulstu.extractor.service.GitRepositoryService;
import ru.ulstu.extractor.service.IndexService;

import java.util.List;

@Controller
public class GitIndexingController {
    private final GitRepositoryService gitRepositoryService;
    private final IndexService indexService;

    public GitIndexingController(GitRepositoryService gitRepositoryService,
                                 IndexService indexService) {
        this.gitRepositoryService = gitRepositoryService;
        this.indexService = indexService;
    }

    @GetMapping("/newRepo")
    public String indexNewRepo(Model model) {
        model.addAttribute(new RepoForm());
        return "newRepo";
    }

    @RequestMapping(value = "/newRepo", method = RequestMethod.POST, params = "send")
    public String getBranch(@ModelAttribute RepoForm repoForm, Model model) {
        try {
            gitRepositoryService.cloneOrUpdateRepo(repoForm.getRepo());
            List<Branch> branches = gitRepositoryService.getBranches(repoForm.getRepo());
            model.addAttribute("branches", branches);
            return "newRepo";
        } catch (Exception ex) {
            model.addAttribute("error", ex.getMessage());
            return "newRepo";
        }
    }

    @RequestMapping(value = "/newRepo", method = RequestMethod.POST, params = "next")
    public String setBranch(@ModelAttribute RepoForm repoForm, Model model, RedirectAttributes redirectAttributes) {
        model.addAttribute("filterForm", new FilterForm(repoForm.getRepo()));
        if (repoForm.getBranch() == null) {
            return "newRepo";
        } else {
            try {
                indexService.index(repoForm.getRepo(), repoForm.getBranch());
            } catch (Exception ex) {
                model.addAttribute("error", ex.getMessage());
                return "newRepo";
            }
            redirectAttributes.addAttribute("repositoryUrl", repoForm.getRepo());
            redirectAttributes.addAttribute("branchName", repoForm.getBranch());
            return "redirect:/filtering";
        }
    }
}
